@@ -25,6 +25,7 @@
 #include "dbspecific.h"
 #include "sqlwchar.h"
 #include <datetime.h>
+#include "wrapper.h"
 
 enum
 {
@@ -959,32 +960,66 @@ static PyObject* Cursor_executemany(PyObject* self, PyObject* args)
         return 0;
     }
 
-    if (!IsSequence(param_seq))
+    if (IsSequence(param_seq))
     {
-        PyErr_SetString(ProgrammingError, "The second parameter to executemany must be a sequence.");
-        return 0;
-    }
+        Py_ssize_t c = PySequence_Size(param_seq);
 
-    Py_ssize_t c = PySequence_Size(param_seq);
-
-    if (c == 0)
-    {
-        PyErr_SetString(ProgrammingError, "The second parameter to executemany must not be empty.");
-        return 0;
-    }
-
-    for (Py_ssize_t i = 0; i < c; i++)
-    {
-        PyObject* params = PySequence_GetItem(param_seq, i);
-        PyObject* result = execute(cursor, pSql, params, false);
-        bool success = result != 0;
-        Py_XDECREF(result);
-        Py_DECREF(params);
-        if (!success)
+        if (c == 0)
         {
-            cursor->rowcount = -1;
+            PyErr_SetString(ProgrammingError, "The second parameter to executemany must not be empty.");
             return 0;
         }
+
+        for (Py_ssize_t i = 0; i < c; i++)
+        {
+            PyObject* params = PySequence_GetItem(param_seq, i);
+            PyObject* result = execute(cursor, pSql, params, false);
+            bool success = result != 0;
+            Py_XDECREF(result);
+            Py_DECREF(params);
+            if (!success)
+            {
+                cursor->rowcount = -1;
+                return 0;
+            }
+        }
+    }
+    else if (PyGen_Check(param_seq) || PyIter_Check(param_seq))
+    {
+        Object iter;
+
+        if (PyGen_Check(param_seq))
+        {
+            iter = PyObject_GetIter(param_seq);
+        }
+        else
+        {
+            iter = param_seq;
+            Py_INCREF(param_seq);
+        }
+
+        Object params;
+
+        while (params.Attach(PyIter_Next(iter)))
+        {
+            PyObject* result = execute(cursor, pSql, params, false);
+            bool success = result != 0;
+            Py_XDECREF(result);
+
+            if (!success)
+            {
+                cursor->rowcount = -1;
+                return 0;
+            }
+        }
+        
+        if (PyErr_Occurred())
+            return 0;
+    }
+    else
+    {
+        PyErr_SetString(ProgrammingError, "The second parameter to executemany must be a sequence, iterator, or generator.");
+        return 0;
     }
 
     cursor->rowcount = -1;
@@ -1040,7 +1075,7 @@ static PyObject* Cursor_fetch(Cursor* cur)
         apValues[i] = value;
     }
 
-    return (PyObject*)Row_New(cur->description, cur->map_name_to_index, field_count, apValues);
+    return (PyObject*)Row_InternalNew(cur->description, cur->map_name_to_index, field_count, apValues);
 }
 
 
@@ -2054,6 +2089,40 @@ static char fetchmany_doc[] =
     "A ProgrammingError exception is raised if the previous call to execute() did\n" \
     "not produce any result set or no call was issued yet.";
 
+
+static char enter_doc[] = "__enter__() -> self.";
+static PyObject* Cursor_enter(PyObject* self, PyObject* args)
+{
+    UNUSED(args);
+    Py_INCREF(self);
+    return self;
+}
+
+static char exit_doc[] = "__exit__(*excinfo) -> None.  Commits the connection if necessary..";
+static PyObject* Cursor_exit(PyObject* self, PyObject* args)
+{
+    Cursor* cursor = Cursor_Validate(self, CURSOR_REQUIRE_OPEN | CURSOR_RAISE_ERROR);
+    if (!cursor)
+        return 0;
+
+    // If an error has occurred, `args` will be a tuple of 3 values.  Otherwise it will be a tuple of 3 `None`s.
+    I(PyTuple_Check(args));
+
+    if (cursor->cnxn->nAutoCommit == SQL_AUTOCOMMIT_OFF && PyTuple_GetItem(args, 0) == Py_None)
+    {
+        SQLRETURN ret;
+        Py_BEGIN_ALLOW_THREADS
+        ret = SQLEndTran(SQL_HANDLE_DBC, cursor->cnxn->hdbc, SQL_COMMIT);
+        Py_END_ALLOW_THREADS
+
+        if (!SQL_SUCCEEDED(ret))
+            return RaiseErrorFromHandle("SQLEndTran(SQL_COMMIT)", cursor->cnxn->hdbc, cursor->hstmt);
+    }
+    
+    Py_RETURN_NONE;
+}
+
+    
 static PyMethodDef Cursor_methods[] =
 {
     { "close",            (PyCFunction)Cursor_close,            METH_NOARGS,                close_doc            },
@@ -2078,6 +2147,8 @@ static PyMethodDef Cursor_methods[] =
     { "skip",             (PyCFunction)Cursor_skip,             METH_VARARGS,               skip_doc             },
     { "commit",           (PyCFunction)Cursor_commit,           METH_NOARGS,                commit_doc           },
     { "rollback",         (PyCFunction)Cursor_rollback,         METH_NOARGS,                rollback_doc         },
+    { "__enter__",        Cursor_enter,                         METH_NOARGS,                enter_doc            },
+    { "__exit__",         Cursor_exit,                          METH_VARARGS,               exit_doc             },
     { 0, 0, 0, 0 }
 };
 
