@@ -3,7 +3,7 @@
 // documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
 // rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 // permit persons to whom the Software is furnished to do so.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 // WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
 // OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
@@ -18,21 +18,19 @@
 #include "errors.h"
 #include "getdata.h"
 #include "cnxninfo.h"
+#include "params.h"
 #include "dbspecific.h"
+#include <datetime.h>
 
 #include <time.h>
 #include <stdarg.h>
 
 static PyObject* MakeConnectionString(PyObject* existing, PyObject* parts);
 
-_typeobject* OurDateTimeType = 0;
-_typeobject* OurDateType = 0;
-_typeobject* OurTimeType = 0;
-
 PyObject* pModule = 0;
 
 static char module_doc[] =
-    "A DB API 2.0 module for ODBC databases.\n"
+    "A database module for accessing databases via ODBC.\n"
     "\n"
     "This module conforms to the DB API 2.0 specification while providing\n"
     "non-standard convenience features.  Only standard Python data types are used\n"
@@ -40,7 +38,11 @@ static char module_doc[] =
     "\n"
     "Static Variables:\n\n"
     "version\n"
-    "  The module version string in the format major.minor.revision\n"
+    "  The module version string.  Official builds will have a version in the format\n"
+    "  `major.minor.revision`, such as 2.1.7.  Beta versions will have -beta appended,\n"
+    "  such as 2.1.8-beta03.  (This would be a build before the official 2.1.8 release.)\n"
+    "  Some special test builds will have a test name (the git branch name) prepended,\n"
+    "  such as fixissue90-2.1.8-beta03.\n"
     "\n"
     "apilevel\n"
     "  The string constant '2.0' indicating this module supports DB API level 2.0.\n"
@@ -50,6 +52,7 @@ static char module_doc[] =
     "  This can be changed any time and affects queries executed after the change.\n"
     "  The default is False.  This can be useful when database columns have\n"
     "  inconsistent capitalization.\n"
+    "\n"
     "pooling\n"
     "  A Boolean indicating whether connection pooling is enabled.  This is a\n"
     "  global (HENV) setting, so it can only be modified before the first\n"
@@ -88,17 +91,17 @@ struct ExcInfo
 #define MAKEEXCINFO(name, parent, doc) { #name, "pyodbc." #name, &name, &parent, doc }
 
 static ExcInfo aExcInfos[] = {
-    MAKEEXCINFO(Error, PyExc_StandardError, 
+    MAKEEXCINFO(Error, PyExc_Exception,
                 "Exception that is the base class of all other error exceptions. You can use\n"
                 "this to catch all errors with one single 'except' statement."),
-    MAKEEXCINFO(Warning, PyExc_StandardError,
+    MAKEEXCINFO(Warning, PyExc_Exception,
                 "Exception raised for important warnings like data truncations while inserting,\n"
                 " etc."),
     MAKEEXCINFO(InterfaceError, Error,
                 "Exception raised for errors that are related to the database interface rather\n"
                 "than the database itself."),
     MAKEEXCINFO(DatabaseError, Error, "Exception raised for errors that are related to the database."),
-    MAKEEXCINFO(DataError, DatabaseError, 
+    MAKEEXCINFO(DataError, DatabaseError,
                 "Exception raised for errors that are due to problems with the processed data\n"
                 "like division by zero, numeric value out of range, etc."),
     MAKEEXCINFO(OperationalError, DatabaseError,
@@ -127,9 +130,7 @@ PyObject* decimal_type;
 
 HENV henv = SQL_NULL_HANDLE;
 
-char chDecimal        = '.';
-char chGroupSeparator = ',';
-char chCurrencySymbol = '$';
+Py_UNICODE chDecimal = '.';
 
 // Initialize the global decimal character and thousands separator character, used when parsing decimal
 // objects.
@@ -151,28 +152,12 @@ static void init_locale_info()
     }
 
     PyObject* value = PyDict_GetItemString(ldict, "decimal_point");
-    if (value && PyString_Check(value) && PyString_Size(value) == 1)
+    if (value)
     {
-        chDecimal = PyString_AsString(value)[0];
-    }
-        
-    value = PyDict_GetItemString(ldict, "thousands_sep");
-    if (value && PyString_Check(value) && PyString_Size(value) == 1)
-    {
-        chGroupSeparator = PyString_AsString(value)[0];
-
-        if (chGroupSeparator == '\0')
-        {
-            // I don't know why, but the default locale isn't setting ','.  We're going to make the assumption that the
-            // most common values are ',' and '.', and we'll take the opposite of the decimal value.
-            chGroupSeparator = (chDecimal == ',') ? '.' : ',';
-        }
-    }
-
-    value = PyDict_GetItemString(ldict, "currency_symbol");
-    if (value && PyString_Check(value) && PyString_Size(value) == 1)
-    {
-        chCurrencySymbol = PyString_AsString(value)[0];
+        if (PyBytes_Check(value) && PyBytes_Size(value) == 1)
+            chDecimal = (Py_UNICODE)PyBytes_AS_STRING(value)[0];
+        if (PyUnicode_Check(value) && PyUnicode_GET_SIZE(value) == 1)
+            chDecimal = PyUnicode_AS_UNICODE(value)[0];
     }
 }
 
@@ -183,25 +168,17 @@ static bool import_types()
     // imported (among other problems).
 
     PyObject* pdt = PyImport_ImportModule("datetime");
-    
+
     if (!pdt)
         return false;
 
     PyDateTime_IMPORT;
-     
-    if (!PyDateTimeAPI)
-    {
-        PyErr_SetString(PyExc_RuntimeError, "Unable to import the datetime module.");
-        return false;
-    }
-    
-    OurDateTimeType = PyDateTimeAPI->DateTimeType;
-    OurDateType     = PyDateTimeAPI->DateType;
-    OurTimeType     = PyDateTimeAPI->TimeType;
 
     Cursor_init();
     CnxnInfo_init();
     GetData_init();
+    if (!Params_init())
+        return false;
 
     PyObject* decimalmod = PyImport_ImportModule("decimal");
     if (!decimalmod)
@@ -209,13 +186,13 @@ static bool import_types()
         PyErr_SetString(PyExc_RuntimeError, "Unable to import decimal");
         return false;
     }
-    
+
     decimal_type = PyObject_GetAttrString(decimalmod, "Decimal");
     Py_DECREF(decimalmod);
 
     if (decimal_type == 0)
         PyErr_SetString(PyExc_RuntimeError, "Unable to import decimal.Decimal.");
-    
+
     return decimal_type != 0;
 }
 
@@ -234,7 +211,7 @@ static bool AllocateEnv()
             return false;
         }
     }
-    
+
     if (!SQL_SUCCEEDED(SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv)))
     {
         Py_FatalError("Can't initialize module pyodbc.  SQLAllocEnv failed.");
@@ -270,13 +247,15 @@ static keywordmap keywordmaps[] =
 static PyObject* mod_connect(PyObject* self, PyObject* args, PyObject* kwargs)
 {
     UNUSED(self);
-    
+
     Object pConnectString = 0;
     int fAutoCommit = 0;
     int fAnsi = 0;              // force ansi
     int fUnicodeResults = 0;
+    int fReadOnly = 0;
+    long timeout = 0;
 
-    int size = args ? PyTuple_Size(args) : 0;
+    Py_ssize_t size = args ? PyTuple_Size(args) : 0;
 
     if (size > 1)
     {
@@ -300,45 +279,56 @@ static PyObject* mod_connect(PyObject* self, PyObject* args, PyObject* kwargs)
         if (!partsdict.IsValid())
             return 0;
 
-        Object unicodeT;            // used to temporarily hold Unicode objects if we have to convert values to unicode
-
         Py_ssize_t pos = 0;
         PyObject* key = 0;
         PyObject* value = 0;
 
+        Object okey; // in case we need to allocate a new key
+
         while (PyDict_Next(kwargs, &pos, &key, &value))
         {
-            // Note: key and value are *borrowed*.
+            if (!Text_Check(key))
+                return PyErr_Format(PyExc_TypeError, "Dictionary items passed to connect must be strings");
 
-            // Check for the two non-connection string keywords we accept.  (If we get many more of these, create something
-            // table driven.  Are we sure there isn't a Python function to parse keywords but leave those it doesn't know?)
-            const char* szKey = PyString_AsString(key);
+            // // Note: key and value are *borrowed*.
+            //
+            // // Check for the two non-connection string keywords we accept.  (If we get many more of these, create something
+            // // table driven.  Are we sure there isn't a Python function to parse keywords but leave those it doesn't know?)
+            // const char* szKey = PyString_AsString(key);
 
-            if (_strcmpi(szKey, "autocommit") == 0)
+            if (Text_EqualsI(key, "autocommit"))
             {
                 fAutoCommit = PyObject_IsTrue(value);
                 continue;
             }
-            if (_strcmpi(szKey, "ansi") == 0)
+            if (Text_EqualsI(key, "ansi"))
             {
                 fAnsi = PyObject_IsTrue(value);
                 continue;
             }
-            if (_strcmpi(szKey, "unicode_results") == 0)
+            if (Text_EqualsI(key, "unicode_results"))
             {
                 fUnicodeResults = PyObject_IsTrue(value);
                 continue;
             }
-        
-            // Anything else must be a string that is appended, along with the keyword to the connection string.
-
-            if (!(PyString_Check(value) || PyUnicode_Check(value)))
-                return PyErr_Format(PyExc_TypeError, "'%s' is not a string or unicode value'", szKey);
-
-            // Map DB API recommended names to ODBC names (e.g. user --> uid).
-            for (int i = 0; i < _countof(keywordmaps); i++)
+            if (Text_EqualsI(key, "timeout"))
             {
-                if (_strcmpi(szKey, keywordmaps[i].oldname) == 0)
+                timeout = PyInt_AsLong(value);
+                if (PyErr_Occurred())
+                    return 0;
+                continue;
+            }
+            if (Text_EqualsI(key, "readonly"))
+            {
+                fReadOnly = PyObject_IsTrue(value);
+                continue;
+            }
+            
+            // Map DB API recommended names to ODBC names (e.g. user --> uid).
+
+            for (size_t i = 0; i < _countof(keywordmaps); i++)
+            {
+                if (Text_EqualsI(key, keywordmaps[i].oldname))
                 {
                     if (keywordmaps[i].newnameObject == 0)
                     {
@@ -352,24 +342,23 @@ static PyObject* mod_connect(PyObject* self, PyObject* args, PyObject* kwargs)
                 }
             }
 
-            if (PyString_Check(value))
-            {
-                unicodeT.Attach(PyUnicode_FromObject(value));
-                if (!unicodeT.IsValid())
-                    return 0;
-                value = unicodeT.Get();
-            }
-
-            if (PyDict_SetItem(partsdict.Get(), key, value) == -1)
+            PyObject* str = PyObject_Str(value); // convert if necessary
+            if (!str)
                 return 0;
 
-            unicodeT.Detach();
+            if (PyDict_SetItem(partsdict.Get(), key, str) == -1)
+            {
+                Py_XDECREF(str);
+                return 0;
+            }
+
+            Py_XDECREF(str);
         }
 
         if (PyDict_Size(partsdict.Get()))
             pConnectString.Attach(MakeConnectionString(pConnectString.Get(), partsdict));
     }
-    
+
     if (!pConnectString.IsValid())
         return PyErr_Format(PyExc_TypeError, "no connection information was passed");
 
@@ -378,16 +367,15 @@ static PyObject* mod_connect(PyObject* self, PyObject* args, PyObject* kwargs)
         if (!AllocateEnv())
             return 0;
     }
-     
-    return (PyObject*)Connection_New(pConnectString.Get(), fAutoCommit != 0, fAnsi != 0, fUnicodeResults != 0);
+
+    return (PyObject*)Connection_New(pConnectString.Get(), fAutoCommit != 0, fAnsi != 0, fUnicodeResults != 0, timeout, fReadOnly != 0);
 }
 
 
-static PyObject*
-mod_datasources(PyObject* self)
+static PyObject* mod_datasources(PyObject* self)
 {
     UNUSED(self);
-  
+
     if (henv == SQL_NULL_HANDLE && !AllocateEnv())
         return 0;
 
@@ -407,83 +395,70 @@ mod_datasources(PyObject* self)
     for (;;)
     {
         Py_BEGIN_ALLOW_THREADS
-        ret = SQLDataSources(henv, SQL_FETCH_NEXT, szDSN,  _countof(szDSN),  &cbDSN, szDesc, _countof(szDesc), &cbDesc);
+        ret = SQLDataSources(henv, nDirection, szDSN,  _countof(szDSN),  &cbDSN, szDesc, _countof(szDesc), &cbDesc);
         Py_END_ALLOW_THREADS
         if (!SQL_SUCCEEDED(ret))
             break;
-        
+
         PyDict_SetItemString(result, (const char*)szDSN, PyString_FromString((const char*)szDesc));
         nDirection = SQL_FETCH_NEXT;
     }
-    
+
     if (ret != SQL_NO_DATA)
     {
         Py_DECREF(result);
         return RaiseErrorFromHandle("SQLDataSources", SQL_NULL_HANDLE, SQL_NULL_HANDLE);
     }
-    
+
     return result;
 }
 
 
-
-static PyObject*
-mod_timefromticks(PyObject* self, PyObject* args)
+static PyObject* mod_timefromticks(PyObject* self, PyObject* args)
 {
     UNUSED(self);
-    
-    time_t t = 0;
-    struct tm* fields;
-
-    // Sigh...  If a float is passed but we ask for a long, we get a deprecation warning printed to the screen instead
-    // of a failure.  Not only is this not documented, it means we can't reliably use PyArg_ParseTuple('l') anywhere!
-    
-    // if (PyArg_ParseTuple(args, "l", &ticks))
 
     PyObject* num;
     if (!PyArg_ParseTuple(args, "O", &num))
         return 0;
-    
-    if (PyInt_Check(num))
-        t = PyInt_AS_LONG(num);
-    else if (PyLong_Check(num))
-        t = PyLong_AsLong(num);
-    else if (PyFloat_Check(num))
-        t = (long)PyFloat_AS_DOUBLE(num);
-    else
-    {
-        PyErr_SetString(PyExc_TypeError, "TimeFromTicks requires a number.");
+
+    if (!PyNumber_Check(num))
+        return PyErr_Format(PyExc_TypeError, "TimeFromTicks requires a number.");
+
+    Object l(PyNumber_Long(num));
+    if (!l)
         return 0;
-    }
-    
-    fields = localtime(&t);
+
+    time_t t = PyLong_AsLong(num);
+    struct tm* fields = localtime(&t);
 
     return PyTime_FromTime(fields->tm_hour, fields->tm_min, fields->tm_sec, 0);
 }
 
-static PyObject*
-mod_datefromticks(PyObject* self, PyObject* args)
+
+static PyObject* mod_datefromticks(PyObject* self, PyObject* args)
 {
     UNUSED(self);
     return PyDate_FromTimestamp(args);
 }
 
-static PyObject*
-mod_timestampfromticks(PyObject* self, PyObject* args)
+
+static PyObject* mod_timestampfromticks(PyObject* self, PyObject* args)
 {
     UNUSED(self);
     return PyDateTime_FromTimestamp(args);
 }
 
+
 static char connect_doc[] =
-    "connect(str, autocommit=False, ansi=False, **kwargs) --> Connection\n"
+    "connect(str, autocommit=False, ansi=False, timeout=0, **kwargs) --> Connection\n"
     "\n"
     "Accepts an ODBC connection string and returns a new Connection object.\n"
     "\n"
     "The connection string will be passed to SQLDriverConnect, so a DSN connection\n"
     "can be created using:\n"
     "\n"
-    "  DSN=DataSourceName;UID=user;PWD=password\n"
+    "  cnxn = pyodbc.connect('DSN=DataSourceName;UID=user;PWD=password')\n"
     "\n"
     "To connect without requiring a DSN, specify the driver and connection\n"
     "information:\n"
@@ -494,48 +469,57 @@ static char connect_doc[] =
     "documentation or the documentation of your ODBC driver for details.\n"
     "\n"
     "The connection string can be passed as the string `str`, as a list of keywords,\n"
-    "or a combination of the two.  Any keywords except autocommit and ansi (see\n"
-    "below) are simply added to the connection string.\n"
+    "or a combination of the two.  Any keywords except autocommit, ansi, and timeout\n"
+    "(see below) are simply added to the connection string.\n"
     "\n"
     "  connect('server=localhost;user=me')\n"
     "  connect(server='localhost', user='me')\n"
     "  connect('server=localhost', user='me')\n"
     "\n"
-    "The DB API recommends the keywords 'user', 'password', and 'host', but these are\n"
-    "not valid ODBC keywords, so these will be converted to 'uid', 'pwd', and\n"
+    "The DB API recommends the keywords 'user', 'password', and 'host', but these\n"
+    "are not valid ODBC keywords, so these will be converted to 'uid', 'pwd', and\n"
     "'server'.\n"
     "\n"
-    "autocommit\n"
+    "Special Keywords\n"
     "\n"
-    "  If False or zero, the default, transactions are created automatically as\n"
-    "  defined in the DB API 2.  If True or non-zero, the connection is put into ODBC\n"
-    "  autocommit mode and statements are committed automatically.\n"
+    "The following specal keywords are processed by pyodbc and are not added to the\n"
+    "connection string.  (If you must use these in your connection string, pass them\n"
+    "as a string, not as keywords.)\n"
     "\n"
-    "ansi\n"
-    "\n"
-    "  By default, pyodbc first attempts to connect using the Unicode version of\n"
-    "  SQLDriverConnectW.  If the driver returns IM001 indicating it does not support\n"
-    "  the Unicode version, the ANSI version is tried.  Any other SQLSTATE is turned\n"
-    "  into an exception.  Setting ansi to true skips the Unicode attempt and only\n"
-    "  connects using the ANSI version.  This is useful for drivers that return the\n"
-    "  wrong SQLSTATE (or if pyodbc is out of date and should support other\n"
-    "  SQLSTATEs).";
-        
-static char timefromticks_doc[] = 
+    "  autocommit\n"
+    "    If False or zero, the default, transactions are created automatically as\n"
+    "    defined in the DB API 2.  If True or non-zero, the connection is put into\n"
+    "    ODBC autocommit mode and statements are committed automatically.\n"
+    "   \n"
+    "  ansi\n"
+    "    By default, pyodbc first attempts to connect using the Unicode version of\n"
+    "    SQLDriverConnectW.  If the driver returns IM001 indicating it does not\n"
+    "    support the Unicode version, the ANSI version is tried.  Any other SQLSTATE\n"
+    "    is turned into an exception.  Setting ansi to true skips the Unicode\n"
+    "    attempt and only connects using the ANSI version.  This is useful for\n"
+    "    drivers that return the wrong SQLSTATE (or if pyodbc is out of date and\n"
+    "    should support other SQLSTATEs).\n"
+    "   \n"
+    "  timeout\n"
+    "    An integer login timeout in seconds, used to set the SQL_ATTR_LOGIN_TIMEOUT\n"
+    "    attribute of the connection.  The default is 0 which means the database's\n"
+    "    default timeout, if any, is used.\n";
+
+static char timefromticks_doc[] =
     "TimeFromTicks(ticks) --> datetime.time\n"
     "\n"
     "Returns a time object initialized from the given ticks value (number of seconds\n"
     "since the epoch; see the documentation of the standard Python time module for\n"
     "details).";
 
-static char datefromticks_doc[] = 
+static char datefromticks_doc[] =
     "DateFromTicks(ticks) --> datetime.date\n"  \
     "\n"                                                                \
     "Returns a date object initialized from the given ticks value (number of seconds\n" \
     "since the epoch; see the documentation of the standard Python time module for\n" \
     "details).";
 
-static char timestampfromticks_doc[] = 
+static char timestampfromticks_doc[] =
     "TimestampFromTicks(ticks) --> datetime.datetime\n"  \
     "\n"                                                                \
     "Returns a datetime object initialized from the given ticks value (number of\n" \
@@ -548,6 +532,53 @@ static char datasources_doc[] =
     "Returns a dictionary mapping available DSNs to their descriptions.";
 
 
+#ifdef PYODBC_LEAK_CHECK
+static PyObject* mod_leakcheck(PyObject* self, PyObject* args)
+{
+    UNUSED(self, args);
+    pyodbc_leak_check();
+    Py_RETURN_NONE;
+}
+#endif
+
+#ifdef WINVER
+static char drivers_doc[] = "drivers() -> [ driver, ... ]\n\nReturns a list of installed drivers";
+
+static PyObject* mod_drivers(PyObject* self, PyObject* args)
+{
+    UNUSED(self, args);
+
+    RegKey key;
+    long ret = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\ODBC\\ODBCINST.INI\\ODBC Drivers", 0, KEY_QUERY_VALUE, &key.hkey);
+    if (ret != ERROR_SUCCESS)
+        return PyErr_Format(PyExc_RuntimeError, "Unable to access the driver list in the registry.  error=%ld", ret);
+
+    Object results(PyList_New(0));
+    DWORD index = 0;
+    char name[255];
+    DWORD length = _countof(name);
+
+    while (RegEnumValue(key, index++, name, &length, 0, 0, 0, 0) == ERROR_SUCCESS)
+    {
+        if (ret != ERROR_SUCCESS)
+            return PyErr_Format(PyExc_RuntimeError, "RegEnumKeyEx failed with error %ld\n", ret);
+
+        PyObject* oname = PyString_FromStringAndSize(name, (Py_ssize_t)length);
+        if (!oname)
+            return 0;
+
+        if (PyList_Append(results.Get(), oname) != 0)
+        {
+            Py_DECREF(oname);
+            return 0;
+        }
+        length = _countof(name);
+    }
+
+    return results.Detach();
+}
+#endif
+
 static PyMethodDef pyodbc_methods[] =
 {
     { "connect",            (PyCFunction)mod_connect,            METH_VARARGS|METH_KEYWORDS, connect_doc },
@@ -555,6 +586,15 @@ static PyMethodDef pyodbc_methods[] =
     { "DateFromTicks",      (PyCFunction)mod_datefromticks,      METH_VARARGS,               datefromticks_doc },
     { "TimestampFromTicks", (PyCFunction)mod_timestampfromticks, METH_VARARGS,               timestampfromticks_doc },
     { "dataSources",        (PyCFunction)mod_datasources,        METH_NOARGS,                datasources_doc },
+
+#ifdef WINVER
+    { "drivers", (PyCFunction)mod_drivers, METH_NOARGS, drivers_doc },
+#endif
+
+#ifdef PYODBC_LEAK_CHECK
+    { "leakcheck", (PyCFunction)mod_leakcheck, METH_NOARGS, 0 },
+#endif
+
     { 0, 0, 0, 0 }
 };
 
@@ -831,7 +871,7 @@ static bool CreateExceptions()
             Py_DECREF(classdict);
             return false;
         }
-        
+
         PyDict_SetItemString(classdict, "__doc__", doc);
         Py_DECREF(doc);
 
@@ -841,7 +881,7 @@ static bool CreateExceptions()
             Py_DECREF(classdict);
             return false;
         }
-        
+
         // Keep a reference for our internal (C++) use.
         Py_INCREF(*info.ppexc);
 
@@ -851,86 +891,114 @@ static bool CreateExceptions()
     return true;
 }
 
-
-PyMODINIT_FUNC
-initpyodbc()
-{
-#ifdef _DEBUG
-    #ifndef Py_REF_DEBUG
-    #error Py_REF_DEBUG not set!
-    #endif
-
-    int grfDebugFlags = _CRTDBG_ALLOC_MEM_DF | _CRTDBG_CHECK_ALWAYS_DF;
-    _CrtSetDbgFlag(grfDebugFlags);
+#if PY_MAJOR_VERSION >= 3
+static struct PyModuleDef moduledef = {
+    PyModuleDef_HEAD_INIT,
+    "pyodbc",                   // m_name
+    module_doc,
+    -1,                         // m_size
+    pyodbc_methods,             // m_methods
+    0,                          // m_reload
+    0,                          // m_traverse
+    0,                          // m_clear
+    0,                          // m_free
+    };
+  #define MODRETURN(v) v
+#else
+  #define MODRETURN(v)
 #endif
 
+PyMODINIT_FUNC
+#if PY_MAJOR_VERSION >= 3
+PyInit_pyodbc()
+#else
+initpyodbc(void)
+#endif
+{
     ErrorInit();
 
     if (PyType_Ready(&ConnectionType) < 0 || PyType_Ready(&CursorType) < 0 || PyType_Ready(&RowType) < 0 || PyType_Ready(&CnxnInfoType) < 0)
-        return;
+        return MODRETURN(0);
 
-    pModule = Py_InitModule4("pyodbc", pyodbc_methods, module_doc, NULL, PYTHON_API_VERSION);
+    Object module;
 
-    if (!import_types())
-        return;
+#if PY_MAJOR_VERSION >= 3
+    module.Attach(PyModule_Create(&moduledef));
+#else
+    module.Attach(Py_InitModule4("pyodbc", pyodbc_methods, module_doc, NULL, PYTHON_API_VERSION));
+#endif
+
+    pModule = module.Get();
+
+    if (!module || !import_types() || !CreateExceptions())
+        return MODRETURN(0);
 
     init_locale_info();
 
-    if (!CreateExceptions())
-        return;
+    const char* szVersion = TOSTRING(PYODBC_VERSION);
+    PyModule_AddStringConstant(module, "version", (char*)szVersion);
 
-    // The 'build' version number is a beta identifier.  For example, if it is 7, then we are on beta7 of the
-    // (major,minor.micro) version.  On Windows, we poke these values into the DLL's version resource, so when we make
-    // an official build (which come *after* the betas), we set the BUILD to 9999 so installers will know that it
-    // should replace any installed betas.  However, we obviously don't want to see these.
-
-    PyObject* pVersion;
-    if (PYODBC_BUILD == 9999)
-        pVersion = PyString_FromFormat("%d.%d.%d", PYODBC_MAJOR, PYODBC_MINOR, PYODBC_MICRO);
-    else
-        pVersion = PyString_FromFormat("%d.%d.%d-beta%d", PYODBC_MAJOR, PYODBC_MINOR, PYODBC_MICRO, PYODBC_BUILD);
-    PyModule_AddObject(pModule, "version", pVersion);
-
-    PyModule_AddIntConstant(pModule, "threadsafety", 1);
-    PyModule_AddStringConstant(pModule, "apilevel", "2.0");
-    PyModule_AddStringConstant(pModule, "paramstyle", "qmark");
-    PyModule_AddObject(pModule, "pooling", Py_True);
+    PyModule_AddIntConstant(module, "threadsafety", 1);
+    PyModule_AddStringConstant(module, "apilevel", "2.0");
+    PyModule_AddStringConstant(module, "paramstyle", "qmark");
+    PyModule_AddObject(module, "pooling", Py_True);
     Py_INCREF(Py_True);
-    PyModule_AddObject(pModule, "lowercase", Py_False);
+    PyModule_AddObject(module, "lowercase", Py_False);
     Py_INCREF(Py_False);
-                       
-    PyModule_AddObject(pModule, "Connection", (PyObject*)&ConnectionType);
+
+    PyModule_AddObject(module, "Connection", (PyObject*)&ConnectionType);
     Py_INCREF((PyObject*)&ConnectionType);
-    PyModule_AddObject(pModule, "Cursor", (PyObject*)&CursorType);
+    PyModule_AddObject(module, "Cursor", (PyObject*)&CursorType);
     Py_INCREF((PyObject*)&CursorType);
-    PyModule_AddObject(pModule, "Row", (PyObject*)&RowType);
+    PyModule_AddObject(module, "Row", (PyObject*)&RowType);
     Py_INCREF((PyObject*)&RowType);
 
     // Add the SQL_XXX defines from ODBC.
     for (unsigned int i = 0; i < _countof(aConstants); i++)
-        PyModule_AddIntConstant(pModule, (char*)aConstants[i].szName, aConstants[i].value);
+        PyModule_AddIntConstant(module, (char*)aConstants[i].szName, aConstants[i].value);
 
-    PyModule_AddObject(pModule, "Date", (PyObject*)PyDateTimeAPI->DateType);
+    PyModule_AddObject(module, "Date", (PyObject*)PyDateTimeAPI->DateType);
     Py_INCREF((PyObject*)PyDateTimeAPI->DateType);
-    PyModule_AddObject(pModule, "Time", (PyObject*)PyDateTimeAPI->TimeType);
+    PyModule_AddObject(module, "Time", (PyObject*)PyDateTimeAPI->TimeType);
     Py_INCREF((PyObject*)PyDateTimeAPI->TimeType);
-    PyModule_AddObject(pModule, "Timestamp", (PyObject*)PyDateTimeAPI->DateTimeType);
+    PyModule_AddObject(module, "Timestamp", (PyObject*)PyDateTimeAPI->DateTimeType);
     Py_INCREF((PyObject*)PyDateTimeAPI->DateTimeType);
-    PyModule_AddObject(pModule, "DATETIME", (PyObject*)PyDateTimeAPI->DateTimeType);
+    PyModule_AddObject(module, "DATETIME", (PyObject*)PyDateTimeAPI->DateTimeType);
     Py_INCREF((PyObject*)PyDateTimeAPI->DateTimeType);
-    PyModule_AddObject(pModule, "STRING", (PyObject*)&PyString_Type);
+    PyModule_AddObject(module, "STRING", (PyObject*)&PyString_Type);
     Py_INCREF((PyObject*)&PyString_Type);
-    PyModule_AddObject(pModule, "NUMBER", (PyObject*)&PyFloat_Type);
+    PyModule_AddObject(module, "NUMBER", (PyObject*)&PyFloat_Type);
     Py_INCREF((PyObject*)&PyFloat_Type);
-    PyModule_AddObject(pModule, "ROWID", (PyObject*)&PyInt_Type);
+    PyModule_AddObject(module, "ROWID", (PyObject*)&PyInt_Type);
     Py_INCREF((PyObject*)&PyInt_Type);
-    PyModule_AddObject(pModule, "BINARY", (PyObject*)&PyBuffer_Type);
-    Py_INCREF((PyObject*)&PyBuffer_Type);
-    PyModule_AddObject(pModule, "Binary", (PyObject*)&PyBuffer_Type);
-    Py_INCREF((PyObject*)&PyBuffer_Type);
-    
-    if (PyErr_Occurred())
+
+    PyObject* binary_type;
+#if PY_VERSION_HEX >= 0x02060000
+    binary_type = (PyObject*)&PyByteArray_Type;
+#else
+    binary_type = (PyObject*)&PyBuffer_Type;
+#endif
+    PyModule_AddObject(module, "BINARY", binary_type);
+    Py_INCREF(binary_type);
+    PyModule_AddObject(module, "Binary", binary_type);
+    Py_INCREF(binary_type);
+
+    I(null_binary != 0);        // must be initialized first
+    PyModule_AddObject(module, "BinaryNull", null_binary);
+
+    PyModule_AddIntConstant(module, "UNICODE_SIZE", sizeof(Py_UNICODE));
+    PyModule_AddIntConstant(module, "SQLWCHAR_SIZE", sizeof(SQLWCHAR));
+
+    if (!PyErr_Occurred())
+    {
+        module.Detach();
+    }
+    else
+    {
         ErrorCleanup();
+    }
+
+    return MODRETURN(pModule);
 }
 
 #ifdef WINVER
@@ -945,14 +1013,26 @@ BOOL WINAPI DllMain(
 }
 #endif
 
+
 static PyObject* MakeConnectionString(PyObject* existing, PyObject* parts)
 {
     // Creates a connection string from an optional existing connection string plus a dictionary of keyword value
-    // pairs.  The keywords must be String objects and the values must be Unicode objects.
+    // pairs.
+    //
+    // existing
+    //   Optional Unicode connection string we will be appending to.  Used when a partial connection string is passed
+    //   in, followed by keyword parameters:
+    //
+    //   connect("driver={x};database={y}", user='z')
+    //
+    // parts
+    //   A dictionary of text keywords and text values that will be appended.
 
-    int length = 0;
+    I(PyUnicode_Check(existing));
+
+    Py_ssize_t length = 0;      // length in *characters*
     if (existing)
-        length = PyUnicode_GET_SIZE(existing) + 1; // + 1 to add a trailing 
+        length = Text_Size(existing) + 1; // + 1 to add a trailing semicolon
 
     Py_ssize_t pos = 0;
     PyObject* key = 0;
@@ -960,35 +1040,29 @@ static PyObject* MakeConnectionString(PyObject* existing, PyObject* parts)
 
     while (PyDict_Next(parts, &pos, &key, &value))
     {
-        length += PyString_GET_SIZE(key) + 1 + PyUnicode_GET_SIZE(value) + 1; // key=value;
+        length += Text_Size(key) + 1 + Text_Size(value) + 1; // key=value;
     }
-    
+
     PyObject* result = PyUnicode_FromUnicode(0, length);
     if (!result)
         return 0;
 
     Py_UNICODE* buffer = PyUnicode_AS_UNICODE(result);
-    int offset = 0;
+    Py_ssize_t offset = 0;
 
     if (existing)
     {
-        memcpy(&buffer[offset], PyUnicode_AS_UNICODE(existing), PyUnicode_GET_SIZE(existing) * sizeof(Py_UNICODE));
-        offset += PyUnicode_GET_SIZE(existing);
+        offset += TextCopyToUnicode(&buffer[offset], existing);
         buffer[offset++] = (Py_UNICODE)';';
     }
 
     pos = 0;
     while (PyDict_Next(parts, &pos, &key, &value))
     {
-        const char* szKey = PyString_AS_STRING(key);
-        for (int i = 0; i < PyString_GET_SIZE(key); i++)
-            buffer[offset++] = (Py_UNICODE)szKey[i];
-
+        offset += TextCopyToUnicode(&buffer[offset], key);
         buffer[offset++] = (Py_UNICODE)'=';
 
-        memcpy(&buffer[offset], PyUnicode_AS_UNICODE(value), PyUnicode_GET_SIZE(value) * sizeof(Py_UNICODE));
-        offset += PyUnicode_GET_SIZE(value);
-        
+        offset += TextCopyToUnicode(&buffer[offset], value);
         buffer[offset++] = (Py_UNICODE)';';
     }
 
